@@ -275,7 +275,7 @@ class EnvVars:
 
         existing_env_var = self.list_for_project(project_id, pprint_result=False)["projectEnvironmentVariables"]
 
-        if name not in [env_var["name"] for env_var in existing_env_var]:
+        if name not in [env_var["name"] for env_var in existing_env_var if env_var["scope"] == "PROJECT"]:
             raise ValueError(f"❌ Environment variable {name} does not exists")
 
         params = [d for d in existing_env_var if d["name"] == name][0]
@@ -323,7 +323,7 @@ class EnvVars:
         """
 
         existing_env_var = self.list_for_project(project_id, pprint_result=False)["projectEnvironmentVariables"]
-        present = name in [env["name"] for env in existing_env_var]
+        present = name in [env["name"] for env in existing_env_var if env["scope"] == "PROJECT"]
 
         # If variable not present, create it
         if not present:
@@ -403,18 +403,24 @@ class EnvVars:
             logging.warning("Cannot get the information of environment variable of the project [%s]", project_id)
             logging.error("Something went wrong %s", exception)
         if project_env_var:
-            for env in project_env_var:
-                env_var_name = env["name"]
-                create_folder(output_folder + env_var_name)
-                write_to_json_file(output_folder + env_var_name + "/variable.json", env)
+            try:
+                for env in project_env_var:
+                    env_var_name = env["name"]
+                    create_folder(output_folder + env_var_name)
+                    write_to_json_file(output_folder + env_var_name + "/variable.json", env)
 
-            logging.info("✅ Environment variables of the project [%s] have been successfully exported", project_id)
+                logging.info("✅ Environment variables of the project [%s] have been successfully exported", project_id)
+            except Exception as exception:
+                logging.warning(
+                    "❌ Environment variables of the project [%s] have not been successfully exported", project_id
+                )
+                logging.error("Something went wrong %s", exception)
+                write_error(error_folder, "env_vars", project_id)
+                result = False
+                return result
         else:
-            logging.warning(
-                "❌ Environment variables of the project [%s] have not been successfully exported", project_id
-            )
-            write_error(error_folder, "env_vars", project_id)
-            result = False
+            logging.info("✅ The project [%s] doesn't have any environment variable", project_id)
+
         return result
 
     def import_from_json(self, json_file: str, project_id: str = None) -> bool:
